#!/usr/bin/perl
#
# written by Assen Roguev, 2006
# questions on the code to: roguev@cmp.ucsf.edu or stoynov@obzor.bio21.bas.bg
#
# thermo_DR.pl
# usage:
#		./thermo_DR.pl input_file start_position stop_position window_size step
# 
# calculates the thermodynamic stability of DNA-RNA duplexes where RNA is equivalent to the Crick coding strand. 
# for given set of nearest-neighbour parameters using a sliding window with arbitrtary size and step
# The values of the nearest-neighbor interactions for the RNA/DNA duplex are reported by Sugimoto et al. (1995) Biochemistry, 34, 11211-11216
# The input file is the Watson DNA coding sequence, therefore TT is written instead of UU.
# this software is freeware
# feel free to distribute and modify it
#

# let the code begin
use strict;

# hash of arrays with thermodynamic parameters
# hash keys are respective pairs
# first array element is enthalpy (dH)
# second array element is enthropy (dS)
my %data_hash = (
'AA' => [11.5, 0.0364],
'TT' => [7.8, 0.0219],
'AT' => [8.3, 0.0239],
'TA' => [7.8, 0.0232],
'CA' => [10.4, 0.0284],
'TG' => [9.0, 0.0261],
'GT' => [5.9, 0.0123],
'AC' => [7.8, 0.0216],
'CT' => [9.1, 0.0235],
'AG' => [7.0, 0.0197],
'GA' => [8.6, 0.0229],
'TC' => [5.5, 0.0135],
'CG' => [16.3, 0.0471],
'GC' => [8.0, 0.0171],
'GG' => [9.3, 0.0232],
'CC' => [12.8, 0.0319]
);

# input parameters for the script
my $seq_file = shift;			# sequence file
my $start = shift;				# calculation start position 
my $stop = shift;				# calculation end position
my $window = shift;				# window size
my $step = shift;				# step

# some variables we are gonna use later
# define them here as globals to avoid scope issues
my $S;				# entropy
my $H;				# entalpy
my $G;				# free energy
my $roundG;			# rounded free energy, used for the output
my $input_seq;		# input sequence after stiching the lines	
my $current_pos;	# current position within the sequence
my $seq;			# working sequence
my $N_character = 'N';

# open the input file
open(IN,$seq_file) || die "Cannot open $seq_file\n";

# read line by line
while (my $line = <IN>)					# read a line
	{
	chomp($line);						# remove new line characters at the end of the lines
	$input_seq = $input_seq.$line;		# stitch the line with the rest
	}
close(IN);								# close input file

# define the working sequence based on start and stop position 
$seq = substr($input_seq, $start, ($stop-$start)); 

# do the maths
for (my $i = 0; $i <= length($seq) - $window; $i+= $step)	# incerement by the given step
	{
	# zero the variables
	$S = 0;
	$H = 0;
	$G = 0;

	# define a subsequnce based on the window size 
	my $sub_seq = substr($seq,$i,$window);

	# removes sequences with Ns in them
	if ($sub_seq =~ /$N_character/) { next; }

	# extract all the nearest neighbour pairs
	for (my $j = 0; $j < length($sub_seq); $j++)
		{
		my $pair = substr($sub_seq,$j,2);
	
		$pair = uc($pair);		# convert to upper case
		
		# get respective values from the hash
		$H += $data_hash{$pair}->[0];
		$S += $data_hash{$pair}->[1]; 
		
		# calculate free energy
		$G =$H*(1-310/($H/$S-36)); 
		}
		
	$roundG = sprintf "%.2f",$G;	# round the value	
	$current_pos = $start + 1 + $i + $window/2;		# calculate where we are now. If window size is an odd number, add 0.5.
	print "$current_pos\t$roundG\n";				# print output on screen
	}